package com.agilecontrol.embed.component;

import cn.lifecycle.cron.annotation.DynamicCron;
import com.agilecontrol.config.Resources;
import com.agilecontrol.config.ResourcesConstant;
import com.agilecontrol.nea.core.schema.ClientManager;
import com.agilecontrol.nea.core.schema.Database;
import com.agilecontrol.nea.core.util.AdSqlUtils;
import com.agilecontrol.nea.core.util.ConfigValues;
import com.agilecontrol.nea.core.util.JsonUtil;
import com.agilecontrol.nea.util.Configurations;
import com.agilecontrol.nea.util.FileUtils;
import com.agilecontrol.nea.util.Validator;
import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.expression.Expression;
import org.springframework.expression.spel.SpelCompilerMode;
import org.springframework.expression.spel.SpelParserConfiguration;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.stereotype.Component;
import org.springframework.util.StopWatch;

@Component
public class DeleteOldFiles {

   private static final Logger log = LoggerFactory.getLogger(DeleteOldFiles.class);


   @DynamicCron(
      desc = "每天删除超期的临时文件",
      cron = "${portal.dynamiccron.daily:0 0 0 * * ?}"
   )
   public void dailyUpdate() {
      StopWatch watch = new StopWatch();
      watch.start();
      int days = ConfigValues.get("portal.oldfile.userhome.expiredays", 7);
      String folder = ConfigValues.DIR_NEA_ROOT + File.separator + "home" + File.separator + ClientManager.getInstance().getDefaultClient().getDomain();
      ArrayList oldPathDefs = new ArrayList();
      DeleteOldFiles.OldPathDef userHomeDef = new DeleteOldFiles.OldPathDef();
      userHomeDef.path = folder;
      userHomeDef.expire = days;
      oldPathDefs.add(userHomeDef);
      DeleteOldFiles.OldPathDef poiPathDef = new DeleteOldFiles.OldPathDef();
      days = ConfigValues.get("portal.oldfile.poifiles.expiredays", 1);
      File folderObj = new File(System.getProperty("java.io.tmpdir"), "poifiles");
      poiPathDef.path = folderObj.getAbsolutePath();
      poiPathDef.expire = days;
      oldPathDefs.add(poiPathDef);
      String oldPathConf = AdSqlUtils.getValue("delete_old_path_conf");
      if(Validator.isNotNull(oldPathConf)) {
         List sum = JsonUtil.readValueToList(oldPathConf, DeleteOldFiles.OldPathDef.class);
         oldPathDefs.addAll(sum);
      }

      int sum1 = oldPathDefs.parallelStream().mapToInt(applyAsInt<invokedynamic>(this)).sum();
      watch.stop();
      log.info("Sum for files deleted: {}, duration:{} seconds", Integer.valueOf(sum1), Double.valueOf(watch.getTotalTimeSeconds()));
   }

   private String parseString(String str) {
      if(!str.contains("#")) {
         return str;
      } else {
         SpelParserConfiguration spelParserConfiguration = new SpelParserConfiguration(SpelCompilerMode.IMMEDIATE, this.getClass().getClassLoader());
         SpelExpressionParser expressionParser = new SpelExpressionParser(spelParserConfiguration);
         StandardEvaluationContext evaluationContext = new StandardEvaluationContext();
         evaluationContext.setVariable("env", ResourcesConstant.environment);
         evaluationContext.setVariable("systemProperties", System.getProperties());
         String localSchema = Database.getInstance().getLocalThreadSchema();
         Configurations conf = Database.getInstance().getConfig(localSchema);
         evaluationContext.setVariable("adParam", conf.getProperties());
         Expression expression = expressionParser.parseExpression(str);
         String result = (String)expression.getValue(evaluationContext, Resources.getApplicationContext(), String.class);
         return result;
      }
   }

   private int deleteFilesOlderThanNdays(int daysBack, String dirWay, boolean includeSub, final String ext) {
      final long purgeTime = System.currentTimeMillis() - (long)(daysBack * 24 * 60 * 60) * 1000L;
      final List extensions = Validator.isNull(ext)?null:(List)Arrays.stream(ext.split(",")).map(apply<invokedynamic>()).collect(Collectors.toList());
      FileFilter filter = new FileFilter() {
         public boolean accept(File f) {
            boolean isAccepted = f.isFile() && f.lastModified() < purgeTime;
            if(isAccepted) {
               if(!Validator.isNull(ext) && !"*".equals(ext)) {
                  String fext = FileUtils.getExtension(f.getName()).trim().toLowerCase();
                  isAccepted = extensions.stream().anyMatch((ext) -> {
                     return e.equals(fext);
                  });
               } else {
                  isAccepted = true;
               }
            }

            return isAccepted;
         }
      };
      FileFilter dirFilter = new FileFilter() {
         public boolean accept(File file) {
            return file.isDirectory();
         }
      };
      File directory = new File(dirWay);
      return this.deleteFiles(directory, filter, dirFilter, includeSub);
   }

   private int deleteFiles(File directory, FileFilter fileFilter, FileFilter dirFilter, boolean inlcudeSub) {
      int cnt = 0;
      if(directory.exists()) {
         File[] listFiles = directory.listFiles(fileFilter);
         File[] dirs = listFiles;
         int var8 = listFiles.length;

         int var9;
         for(var9 = 0; var9 < var8; ++var9) {
            File listFile = dirs[var9];
            cnt += listFile.delete()?1:0;
         }

         if(inlcudeSub) {
            dirs = directory.listFiles(dirFilter);
            File[] var13 = dirs;
            var9 = dirs.length;

            for(int var14 = 0; var14 < var9; ++var14) {
               File dir = var13[var14];
               cnt += this.deleteFiles(dir, fileFilter, dirFilter, inlcudeSub);
               File[] files = dir.listFiles();
               if(files.length == 0) {
                  dir.delete();
               }
            }
         }
      } else {
         log.warn("Files were not deleted, directory " + directory.getPath() + " does\'nt exist!");
      }

      return cnt;
   }

   // $FF: synthetic method
   private static String lambda$deleteFilesOlderThanNdays$1(String e) {
      return e.trim().toLowerCase();
   }

   // $FF: synthetic method
   private int lambda$dailyUpdate$0(DeleteOldFiles.OldPathDef oldPathDef) {
      String path = this.parseString(oldPathDef.path);
      int cnt = this.deleteFilesOlderThanNdays(oldPathDef.expire, path, true, oldPathDef.ext);
      log.debug("Total {} files deleted in {} older than {} days, for ext {}", new Object[]{Integer.valueOf(cnt), path, Integer.valueOf(oldPathDef.expire), oldPathDef.ext});
      return cnt;
   }


   static class OldPathDef {

      String path;
      String ext;
      int expire;


      public String getPath() {
         return this.path;
      }

      public String getExt() {
         return this.ext;
      }

      public int getExpire() {
         return this.expire;
      }

      public void setPath(String path) {
         this.path = path;
      }

      public void setExt(String ext) {
         this.ext = ext;
      }

      public void setExpire(int expire) {
         this.expire = expire;
      }

      public boolean equals(Object o) {
         if(o == this) {
            return true;
         } else if(!(o instanceof DeleteOldFiles.OldPathDef)) {
            return false;
         } else {
            DeleteOldFiles.OldPathDef other = (DeleteOldFiles.OldPathDef)o;
            if(!other.canEqual(this)) {
               return false;
            } else {
               label39: {
                  String this$path = this.getPath();
                  String other$path = other.getPath();
                  if(this$path == null) {
                     if(other$path == null) {
                        break label39;
                     }
                  } else if(this$path.equals(other$path)) {
                     break label39;
                  }

                  return false;
               }

               String this$ext = this.getExt();
               String other$ext = other.getExt();
               if(this$ext == null) {
                  if(other$ext != null) {
                     return false;
                  }
               } else if(!this$ext.equals(other$ext)) {
                  return false;
               }

               if(this.getExpire() != other.getExpire()) {
                  return false;
               } else {
                  return true;
               }
            }
         }
      }

      protected boolean canEqual(Object other) {
         return other instanceof DeleteOldFiles.OldPathDef;
      }

      public int hashCode() {
         boolean PRIME = true;
         byte result = 1;
         String $path = this.getPath();
         int result1 = result * 59 + ($path == null?43:$path.hashCode());
         String $ext = this.getExt();
         result1 = result1 * 59 + ($ext == null?43:$ext.hashCode());
         result1 = result1 * 59 + this.getExpire();
         return result1;
      }

      public String toString() {
         return "DeleteOldFiles.OldPathDef(path=" + this.getPath() + ", ext=" + this.getExt() + ", expire=" + this.getExpire() + ")";
      }
   }
}
